From: Evangelos Petrongonas <epetron@amazon.de>
To: <ardb@kernel.org>
Cc: Evangelos Petrongonas <epetron@amazon.de>,
Alexander Graf <graf@amazon.com>, Mike Rapoport <rppt@kernel.org>,
Changyuan Lyu <changyuanl@google.com>,
<kexec@lists.infradead.org>, <nh-open-source@amazon.com>,
<linux-efi@vger.kernel.org>, <linux-kernel@vger.kernel.org>
Subject: [PATCH] efi: Support booting with kexec handover (KHO)
Date: Fri, 8 Aug 2025 16:36:51 +0000 [thread overview]
Message-ID: <20250808163651.25279-1-epetron@amazon.de> (raw)
When KHO (Kexec HandOver) is enabled, it sets up scratch memory regions
early during device tree scanning. After kexec, the new kernel
exclusively uses this region for memory allocations during boot up to
the initialization of the page allocator
However, when booting with EFI, EFI's reserve_regions() uses
memblock_remove(0, PHYS_ADDR_MAX) to clear all memory regions before
rebuilding them from EFI data. This destroys KHO scratch regions and
their flags, thus causing a kernel panic, as there are no scratch
memory regions.
Instead of wholesale removal, iterate through memory regions and only
remove non-KHO ones. This preserves KHO scratch regions while still
allowing EFI to rebuild its memory map.
Signed-off-by: Evangelos Petrongonas <epetron@amazon.de>
---
Reproduction/Verification Steps
The issue and the fix can be reproduced/verified by booting a VM with
EFI and attempting to perform a KHO enabled kexec. The fix
was developed/tested on arm64.
drivers/firmware/efi/efi-init.c | 31 +++++++++++++++++++++++++++----
1 file changed, 27 insertions(+), 4 deletions(-)
diff --git a/drivers/firmware/efi/efi-init.c b/drivers/firmware/efi/efi-init.c
index a00e07b853f22..2f08b1ab764f6 100644
--- a/drivers/firmware/efi/efi-init.c
+++ b/drivers/firmware/efi/efi-init.c
@@ -164,12 +164,35 @@ static __init void reserve_regions(void)
pr_info("Processing EFI memory map:\n");
/*
* Discard memblocks discovered so far: if there are any at this
* point, they originate from memory nodes in the DT, and UEFI
* uses its own memory map instead.
*/
memblock_dump_all();
- memblock_remove(0, PHYS_ADDR_MAX);
+
+ if (IS_ENABLED(CONFIG_MEMBLOCK_KHO_SCRATCH)) {
+ struct memblock_region *reg;
+ phys_addr_t start, size;
+ int i;
+
+ /* Remove all non-KHO regions */
+ for (i = memblock.memory.cnt - 1; i >= 0; i--) {
+ reg = &memblock.memory.regions[i];
+ if (!memblock_is_kho_scratch(reg)) {
+ start = reg->base;
+ size = reg->size;
+ memblock_remove(start, size);
+ }
+ }
+ } else {
+ /*
+ * KHO is disabled. Discard memblocks discovered so far: if there
+ * are any at this point, they originate from memory nodes in the
+ * DT, and UEFI uses its own memory map instead.
+ */
+ memblock_remove(0, PHYS_ADDR_MAX);
+ }
for_each_efi_memory_desc(md) {
paddr = md->phys_addr;
@@ -164,12 +164,35 @@ static __init void reserve_regions(void)
pr_info("Processing EFI memory map:\n");
/*
- * Discard memblocks discovered so far: if there are any at this
- * point, they originate from memory nodes in the DT, and UEFI
- * uses its own memory map instead.
+ * Discard memblocks discovered so far except for KHO scratch regions.
+ * Most memblocks at this point originate from memory nodes in the DT,
+ * and UEFI uses its own memory map instead. However, if KHO is enabled,
+ * scratch regions must be preserved.
*/
memblock_dump_all();
--
2.43.0
Amazon Web Services Development Center Germany GmbH
Tamara-Danz-Str. 13
10243 Berlin
Geschaeftsfuehrung: Christian Schlaeger, Jonathan Weiss
Eingetragen am Amtsgericht Charlottenburg unter HRB 257764 B
Sitz: Berlin
Ust-ID: DE 365 538 597
next reply other threads:[~2025-08-08 16:37 UTC|newest]
Thread overview: 4+ messages / expand[flat|nested] mbox.gz Atom feed top
2025-08-08 16:36 Evangelos Petrongonas [this message]
2025-08-11 6:39 ` Mike Rapoport
2025-08-14 0:53 ` Evangelos Petrongonas
2025-08-14 8:51 ` Mike Rapoport
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20250808163651.25279-1-epetron@amazon.de \
--to=epetron@amazon.de \
--cc=ardb@kernel.org \
--cc=changyuanl@google.com \
--cc=graf@amazon.com \
--cc=kexec@lists.infradead.org \
--cc=linux-efi@vger.kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=nh-open-source@amazon.com \
--cc=rppt@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®